@@ -2,6 +2,7 @@
 package jpl.test;
 
 import java.util.Map;
+
 import jpl.Atom;
 import jpl.Compound;
 import jpl.Integer;
@@ -18,7 +19,15 @@ import junit.framework.TestSuite;
 // This class defines all the tests which are run from Java.
 // It needs junit.framework.TestCase and junit.framework.TestSuite, which are not supplied with JPL.
 public class TestJUnit extends TestCase {
-	//
+	public static long fac(long n) { // complements jpl:jpl_test_fac(+integer,-integer)
+		if (n == 1) {
+			return 1;
+		} else if (n > 1) {
+			return n * ((jpl.Integer) Query.oneSolution("jpl_test_fac(?,F)", new Term[] { new jpl.Integer(n - 1) }).get("F")).longValue();
+		} else {
+			return 0;
+		}
+	}
 	public TestJUnit(String name) {
 		super(name);
 	}
@@ -30,8 +39,8 @@ public class TestJUnit extends TestCase {
 	}
 	protected void setUp() {
 		// initialization code
-		// Prolog.set_default_init_args(new String[] { "libpl.dll", "-f", "none", "-g", "set_prolog_flag(debug_on_error,false)", "-q" });
-		Prolog.set_default_init_args(new String[] { "libpl.dll", "-f", "none", "-g", "true", "-q" });
+		// Prolog.set_default_init_args(new String[] { "swipl.dll", "-f", "none", "-g", "set_prolog_flag(debug_on_error,false)", "-q" });
+		Prolog.set_default_init_args(new String[] { "swipl.dll", "-f", "none", "-g", "true", "-q" });
 		assertTrue((new Query("consult(test_jpl)")).hasSolution());
 	}
 	protected void tearDown() {
@@ -40,7 +49,6 @@ public class TestJUnit extends TestCase {
 	//
 	public void testMasstest() {
 		assertTrue((new Query("assert(diagnose_declaration(_,_,_,[not,a,real,error]))")).hasSolution());
-
 	}
 	public void testSameLibVersions1() {
 		String java_lib_version = JPL.version_string();
@@ -214,9 +222,9 @@ public class TestJUnit extends TestCase {
 	}
 	private void testMutualRecursion(int n, long f) { // f is the expected result for fac(n)
 		try {
-			assertEquals("mutual recursive Java<->Prolog factorial: jpl.test.Test.fac(" + n + ") = " + f, jpl.test.Test.fac(n), f);
+			assertEquals("mutual recursive Java<->Prolog factorial: fac(" + n + ") = " + f, fac(n), f);
 		} catch (Exception e) {
-			fail("jpl.test.Test.fac(" + n + ") threw " + e);
+			fail("fac(" + n + ") threw " + e);
 		}
 	}
 	public void testMutualRecursion1() {
@@ -436,11 +444,9 @@ public class TestJUnit extends TestCase {
 	//	public void testMaxInteger1(){
 	//		assertEquals(((Term)(Query.oneSolution("current_prolog_flag(max_integer,I)").get("I"))).longValue(), java.lang.Long.MAX_VALUE); // i.e. 9223372036854775807L
 	//	}
-	public void testSingleton1() {
-		// Query.hasSolution("consult('jpl/test/testSingleton1.pl')");
-		// Query.hasSolution("load_files('jpl/test/testSingleton1.pl',[silent(true)])");
-		assertTrue(Query.hasSolution("style_check(-singleton),consult('test_singleton.pl')"));
-	}
+	//	public void testSingleton1() {
+	//		assertTrue(Query.hasSolution("style_check(-singleton),consult('test_singleton.pl')"));
+	//	}
 	public void testStaticQueryInvalidSourceText2() {
 		String goal = "p(]";
 		try {
@@ -472,17 +478,17 @@ public class TestJUnit extends TestCase {
 	public void testStaticQueryNSolutions1() {
 		String goal = "member(X, [0,1,2,3,4,5,6,7,8,9])";
 		int n = 5;
-		assertTrue("Query.nSolutions(" + goal + ", " + n + ") returns " + n + " solutions", Query.nSolutions(goal,n).length == n);
+		assertTrue("Query.nSolutions(" + goal + ", " + n + ") returns " + n + " solutions", Query.nSolutions(goal, n).length == n);
 	}
 	public void testStaticQueryNSolutions2() {
 		String goal = "member(X, [0,1,2,3,4,5,6,7,8,9])";
 		int n = 0;
-		assertTrue("Query.nSolutions(" + goal + ", " + n + ") returns " + n + " solutions", Query.nSolutions(goal,n).length == n);
+		assertTrue("Query.nSolutions(" + goal + ", " + n + ") returns " + n + " solutions", Query.nSolutions(goal, n).length == n);
 	}
 	public void testStaticQueryNSolutions3() {
 		String goal = "member(X, [0,1,2,3,4,5,6,7,8,9])";
 		int n = 20;
-		assertTrue("Query.nSolutions(" + goal + ", " + n + ") returns 10 solutions", Query.nSolutions(goal,n).length == 10);
+		assertTrue("Query.nSolutions(" + goal + ", " + n + ") returns 10 solutions", Query.nSolutions(goal, n).length == 10);
 	}
 	public void testStaticQueryAllSolutions1() {
 		String goal = "member(X, [0,1,2,3,4,5,6,7,8,9])";
@@ -490,12 +496,12 @@ public class TestJUnit extends TestCase {
 	}
 	public void testStaticQueryHasSolution1() {
 		String goal = "memberchk(13, [?,?,?])";
-		Term[] params = new Term[] {new Integer(12), new Integer(13), new Integer(14)};
+		Term[] params = new Term[] { new Integer(12), new Integer(13), new Integer(14) };
 		assertTrue(Query.hasSolution(goal, params));
 	}
 	public void testStaticQueryHasSolution2() {
 		String goal = "memberchk(23, [?,?,?])";
-		Term[] params = new Term[] {new Integer(12), new Integer(13), new Integer(14)};
+		Term[] params = new Term[] { new Integer(12), new Integer(13), new Integer(14) };
 		assertFalse(Query.hasSolution(goal, params));
 	}
 	public void testUtilListToTermArray1() {
@@ -510,4 +516,161 @@ public class TestJUnit extends TestCase {
 		Term[] array = list.toTermArray();
 		assertTrue(array[2].isAtom() && array[2].name().equals("c"));
 	}
+	public void testJRef1() {
+		// System.out.println("java.library.path=" + System.getProperties().get("java.library.path"));
+		// System.out.println("jpl.c version = " + jpl.fli.Prolog.get_c_lib_version());
+		int i = 76543;
+		Integer I = new Integer(i);
+		Query q = new Query("jpl_call(?,intValue,[],I2)", new Term[] { Term.objectToJRef(I) });
+		Term I2 = (Term) q.oneSolution().get("I2");
+		assertTrue(I2.isInteger() && I2.intValue() == i);
+	}
+	public void testBerhhard1() {
+		assertTrue(Query.allSolutions( "consult(library('lists'))" ).length == 1);
+	}
+	public void testJRef2() {
+		int i = 76543;
+		Integer I = new Integer(i);
+		Query q = new Query("jpl_call(?,intValue,[],I2)", jpl.JPL.newJRef(I));
+		Term I2 = (Term) q.oneSolution().get("I2");
+		assertTrue(I2.isInteger() && I2.intValue() == i);
+	}
+	public void testJRef3() {
+		StringBuffer sb = new StringBuffer();
+		Query.oneSolution("jpl_call(?,append,['xyz'],_)", new Term[] {JPL.newJRef(sb)});
+		assertTrue(sb.toString().equals("xyz"));
+	}
+	public void testJRef4() {
+		Term jrefSB = (Term) Query.oneSolution("jpl_new('java.lang.StringBuffer',['abc'],SB)").get("SB");
+		assertTrue(jrefSB.isJRef() && ((StringBuffer) jrefSB.jrefToObject()).toString().equals("abc"));
+	}
+	public void testJRef5() {
+		String token = "foobar345";
+		Term a = (Term) (Query.oneSolution("jpl_new('java.lang.StringBuffer',[?],A)", new Term[] {new Atom(token)}).get("A"));
+		assertTrue(((java.lang.StringBuffer) (a.jrefToObject())).toString().equals(token));
+	}
+	public void testRef6() {
+		Term nullJRef = new Compound("@", new Term[] {new Atom("null")});
+		Object nullObject = nullJRef.jrefToObject();
+		assertNull("@(null) .jrefToObject() yields null", nullObject);
+	}
+	public void testRef7() {
+		Term badJRef = new Compound("@", new Term[] {new Atom("foobar")});
+		try {
+			badJRef.jrefToObject();
+			// shouldn't get to here
+			fail("@(foobar) .jrefToObject() shoulda thrown JPLException");
+		} catch (jpl.JPLException e) {
+			// correct exception class, but is it correct in detail?
+			if (e.getMessage().endsWith("term is not a JRef")) {
+				// OK: an appropriate exception was thrown
+			} else {
+				fail("@(foobar) .jrefToObject() threw incorrect JPLException: " + e);
+			}
+		} catch (Exception e) {
+			fail("@(foobar) .jrefToObject() threw wrong class of exception: " + e);
+		}
+	}
+	public void testForeignFrame1() {
+		int ls1 = ((Term) (Query.oneSolution("statistics(localused,LS)").get("LS"))).intValue();
+		int ls2 = ((Term) (Query.oneSolution("statistics(localused,LS)").get("LS"))).intValue();
+		assertTrue("local stack size unchanged after query", ls1 == ls2);
+	}
+	public void testOpenGetClose1() {
+		StringBuffer sb = new StringBuffer();
+		Query q = new Query("atom_chars(prolog, Cs), member(C, Cs)");
+		Map soln;
+		q.open();
+		while ((soln = q.getSolution()) != null) {
+			sb.append(((Atom) soln.get("C")).name());
+		}
+		q.close();
+		assertEquals("prolog", sb.toString());
+	}
+	public void testOpenGetClose2() {
+		Query q = new Query("dummy"); // we're not going to open this...
+		try {
+			q.getSolution(); // but mistakenly try to get a solution from it...
+		} catch (jpl.JPLException e) { // correct exception class, but is it correct in detail?
+			if (e.getMessage().endsWith("Query is not open")) { // ...which should throw a JPLException like this
+				// OK: an appropriate exception was thrown
+			} else {
+				fail("jpl.Query#getSolution() threw incorrect JPLException: " + e);
+			}
+		} catch (Exception e) {
+			fail("jpl.Query#getSolution() threw wrong class of exception: " + e);
+		}
+	}
+	public void testOpen1() {
+		Query q = new Query("dummy");
+		assertTrue("a newly created query is not open", !q.isOpen());
+	}
+	public void testOpen2() {
+		Query q = new Query("fail");
+		q.open();
+		assertTrue("a newly opened query which has no solutions is open", q.isOpen());
+	}
+	public void testGetSolution1() {
+		Query q = new Query("fail");
+		q.open();
+		q.getSolution();
+		assertTrue("an opened query on which getSolution has failed once is closed", !q.isOpen());
+	}
+	public void testGetSolution2() {
+		Query q = new Query("fail"); // this query has no solutions
+		q.open(); // this opens the query
+		q.getSolution(); // this finds no solution, and closes the query
+		try {
+			q.getSolution(); // this call is invalid, as the query is closed
+			// shouldn't get to here
+			fail("jpl.Query#getSolution() shoulda thrown JPLException");
+		} catch (jpl.JPLException e) { // correct exception class, but is it correct in detail?
+			if (e.getMessage().endsWith("Query is not open")) { // ...which should throw a JPLException like this
+				// OK: an appropriate exception was thrown
+			} else {
+				fail("jpl.Query#getSolution() threw incorrect JPLException: " + e);
+			}
+		} catch (Exception e) {
+			fail("jpl.Query#getSolution() threw wrong class of exception: " + e);
+		}
+	}
+	public void testHasMoreSolutions1() {
+		StringBuffer sb = new StringBuffer();
+		Query q = new Query("atom_chars(prolog, Cs), member(C, Cs)");
+		Map soln;
+		q.open();
+		while (q.hasMoreSolutions()) {
+			soln = q.nextSolution();
+			sb.append(((Atom) soln.get("C")).name());
+		}
+		q.close();
+		assertEquals("Query#hasMoreSolutions() + Query#nextSolution() work as intended", "prolog", sb.toString());
+	}
+	public void testHasMoreElements1() {
+		StringBuffer sb = new StringBuffer();
+		Query q = new Query("atom_chars(prolog, Cs), member(C, Cs)");
+		Map soln;
+		q.open();
+		while (q.hasMoreElements()) {
+			soln = (Map) q.nextElement();
+			sb.append(((Atom) soln.get("C")).name());
+		}
+		q.close();
+		assertEquals("Query#hasMoreElements() + Query#nextElement() work as intended", "prolog", sb.toString());
+	}
+	public void testStackedQueries1() {
+		StringBuffer sb = new StringBuffer();
+		Query q = new Query("atom_chars(prolog, Cs), member(C, Cs)");
+		Map soln;
+		q.open();
+		while ((soln = q.getSolution()) != null) {
+			Atom a = (Atom) soln.get("C");
+			if (Query.hasSolution("memberchk(?, [l,o,r])", new Term[] {a})) { // this query opens and closes while an earlier query is still open
+				sb.append(((Atom) soln.get("C")).name());
+			}
+		}
+		assertTrue(!q.isOpen()); // q will have been closed by the final getSolution()
+		assertEquals("rolo", sb.toString());
+	}
+
 }
