@@ -1,10 +1,32 @@
-/*  $Id: texinfo.pl,v 1.4 2002/07/04 07:47:03 jan Exp $
+/*  $Id$
 
-    Part of XPCE
-    Designed and implemented by Anjo Anjewierden and Jan Wielemaker
-    E-mail: jan@swi.psy.uva.nl
+    Part of XPCE --- The SWI-Prolog GUI toolkit
 
-    Copyright (C) 1993 University of Amsterdam. All rights reserved.
+    Author:        Jan Wielemaker and Anjo Anjewierden
+    E-mail:        J.Wielemaker@cs.vu.nl
+    WWW:           http://www.swi-prolog.org/packages/xpce/
+    Copyright (C): 2010, University of Amsterdam
+
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
+    as published by the Free Software Foundation; either version 2
+    of the License, or (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+    As a special exception, if you link this library with other files,
+    compiled with a Free Software compiler, to produce an executable, this
+    library does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however
+    invalidate any other reasons why the executable file might be covered by
+    the GNU General Public License.
 */
 
 :- module(texinfo,
@@ -89,7 +111,7 @@ class_inheritance(Class) :-
 	;   true
 	),
 	output('\n').
-	       
+
 
 show_inheritance(@nil) :- !,
 	output('\n').
@@ -138,7 +160,7 @@ texi_description(Obj, Description) :-
 	Obj = @Ref,
 	assert(fetched_description(Ref, S0)),
 	Description = S0.
-	    
+
 
 :- pce_extend_class(object).
 
@@ -183,7 +205,7 @@ substitute(S, [Search, Replace | Rest]) :-
 	),
 	done_regex(Search, Re),
 	substitute(S, Rest).
-	
+
 
 texinfo_description(S) :-
 	send(S, ensure_nl),
@@ -227,7 +249,7 @@ texinfo_lists(S) :-
 		     '\n+\\(\\(\n+\t.*\\)+\\)',
 		     	call(example)
 		   ]).
-	
+
 
 example(Re, String) :-
 	get(Re, register_value, String, 1, S2),
@@ -261,7 +283,7 @@ list(itemize, Re, String) :-
 	send(S0, ensure_nl),
 	send(S0, prepend, string('@item\n')),
 	send(Re, register_value, String, S0).
-	
+
 
 itemize(Re, String) :-
 	get(Re, register_value, String, 1, S2),
@@ -286,7 +308,7 @@ description(Re, String) :-
 	send(S2, append, string('@end itemize\n\n')),
 	substitute(S2, ['@\\*\n@\\(item\\|end\\)', '\n@\\1']),
 	send(Re, register_value, String, S2).
-	
+
 
 enumerate(Re, String) :-
 	get(Re, register_value, String, 1, S2),
@@ -298,7 +320,7 @@ enumerate(Re, String) :-
 	send(S2, prepend, string('\n\n@enumerate\n')),
 	send(S2, append, string('@end enumerate\n\n')),
 	send(Re, register_value, String, S2).
-	
+
 
 table(Re, String) :-
 	get(Re, register_value, String, 1, S2),
@@ -390,7 +412,7 @@ behaviour(Class) :-
 	group_objects(Chain, Groups),
 	send(Groups, for_all,
 	     message(@prolog, texinfo_group, Class, @arg1?name, @arg1?value)).
-	
+
 
 collect_behaviour(Class, Behaviour) :-
 	new(Behaviour, chain),
@@ -901,7 +923,7 @@ includeonly(F, String) :-
 	     message(C, append,
 		     ?(@file_name_regex, register_value, String)?value)),
 	send(F, attribute, attribute(includeonly, C)).
-		       
+
 include(F, Name) :-
 	(   get(F, attribute, includeonly, Only),
 	    (   (   send(Only, member, Name)
@@ -988,7 +1010,7 @@ class_pattern(File, Class,
 class_pattern(File, Class,
 	      '#description\\s *$',
 	      class_description(File, Class)).
-	      
+
 class_exclude(Class, Type, Name) :-
 	get(Class, Type, Name, Method),
 	get(@document, classes, Classes),
@@ -1177,7 +1199,7 @@ check_documented_all_classes(List) :-
 	    send(AllClasses, for_all,
 		 message(@pce, format, '\t%s\n', @arg1?name))
 	).
-	
+
 
 
 		 /*******************************
@@ -1216,7 +1238,7 @@ make_preview_view(V) :-
 	send(D, append, button(quit,
 			       message(D?frame, destroy))),
 	send(V, open).
-	
+
 
 
 generate(V, Classes) :-
