@@ -32,6 +32,7 @@
 :- module(pce_prolog_tracer,
 	  [ prolog_show_frame/2		% +Frame, +Options
 	  ]).
+:- use_module(library(pce)).
 :- use_module(library(prolog_clause)).
 :- use_module(library(lists)).
 :- use_module(library(debug)).
@@ -76,12 +77,9 @@ user:prolog_trace_interception(Port, Frame, CHP, Action) :-
 
 map_action(creep, _, continue) :-
 	traceall.
-map_action(skip, _, continue) :-
-	get_tracer(selected_frame, Frame),
-	prolog_frame_attribute(Frame, level, Level),
-	Skip is Level + 1,
+map_action(skip, Frame, continue) :-
 	trace,
-	prolog_skip_level(_, Skip).
+	prolog_skip_frame(Frame).
 map_action(into, _, continue) :-
 	visible(+unify),
 	traceall.
@@ -94,16 +92,13 @@ map_action(retry, _, retry(Frame)) :-
 map_action(fail, _, fail) :-
 	traceall.
 map_action(nodebug, _, nodebug).
-map_action(leap, _, continue) :-
-	notrace.
 map_action(abort, _, abort).
 map_action(halt, _, continue) :-
 	halt.
 map_action(finish, _, continue) :-
 	get_tracer(selected_frame, Frame),
-	prolog_frame_attribute(Frame, level, Level),
 	trace,
-	prolog_skip_level(_, Level).
+	prolog_skip_frame(Frame).
 
 %%	traceall is det.
 %
@@ -193,6 +188,7 @@ do_intercept(redo, Frame, CHP, Action) :-
 	prolog_frame_attribute(Frame, goal, Goal),
 	(   predicate_property(Goal, nodebug)
 	;   predicate_property(Goal, foreign)
+	;   prolog_skip_level(redo_in_skip, redo_in_skip)
 	), !,
 	show(Frame, CHP, 1, redo),
 	action(Action).
@@ -275,13 +271,13 @@ show(StartFrame, CHP, Up, Port) :-
 	send_tracer(report(status, '%s: %s', Port?label_name, Pred)).
 
 show(StartFrame, CHP, Up, Port, Style) :-
-	send_tracer(current_frame(StartFrame)),
+	find_frame(Up, StartFrame, Port, PC, Frame),
+	send_tracer(trapped_location(StartFrame, Frame, Port)),
 	prolog_show_frame(StartFrame,
 			  [ port(Port),
 			    choice(CHP),
 			    stack
 			  ]),
-	find_frame(Up, StartFrame, Port, PC, Frame),
 	prolog_show_frame(Frame,
 			  [ pc(PC),
 			    port(Port),
@@ -291,7 +287,7 @@ show(StartFrame, CHP, Up, Port, Style) :-
 			  ]).
 
 
-%%	find_frame(+Up, +StartFrame, +Port, -PC, -Frame)
+%%	find_frame(+Up, +StartFrame, +Port, -PC, -Frame) is det.
 %
 %	Find the parent frame Up levels above StartFrame. Must be called
 %	in the context of the debugged thread.
@@ -354,6 +350,8 @@ tracer_gui(_, GUI) :-
 %	provided by Attributes.  Defined attributes:
 %
 %		* pc(PC)
+%		Location.  This is one of an integer (Program Counter),
+%		a port-name or choice(CHP).
 %		* choice(CHP)
 %		* port(Port)
 %		* style(Style)
@@ -393,26 +391,27 @@ show_source(Frame, Attributes) :-
 	    attribute(Attributes, port(Port), call),
 	    attribute(Attributes, style(Style), Port),
 	    debug('Show source, PC = ~w, Port = ~w~n', [PC, Port]),
-	    (   atom(PC), PC \== exit		% PC is a port-name
-	    ->  prolog_frame_attribute(GUI, Frame, goal, Goal),
-		find_source(Goal, File, Line),
-		debug('At ~w:~d~n', [File, Line]),
-		send_tracer(GUI, show_line(File, Line, Style))
-	    ;   (   prolog_frame_attribute(GUI, Frame, clause, ClauseRef),
-		    debug('ClauseRef = ~w, PC = ~w~n', [ClauseRef, PC]),
-		    ClauseRef \== 0
-		->  subgoal_position(ClauseRef, PC, File, CharA, CharZ),
-		    debug('~p.~n', [show_range(File, CharA, CharZ, Style)]),
-		    send_tracer(GUI, show_range(File, CharA, CharZ, Style)),
-		    (	clause_property(ClauseRef, erased)
-		    ->	send_tracer(GUI,
-				    report(warning,
-					   'Running erased clause; \
-					   source location may be incorrect'))
-		    ;	true
-		    )
-		;   prolog_frame_attribute(GUI, Frame, goal, Goal),
-		    find_source(Goal, File, Line),
+	    (	clause_position(PC),
+		prolog_frame_attribute(GUI, Frame, clause, ClauseRef),
+		debug('ClauseRef = ~w, PC = ~w~n', [ClauseRef, PC]),
+		ClauseRef \== 0
+	    ->	subgoal_position(GUI, ClauseRef, PC, File, CharA, CharZ),
+		debug('~p.~n', [show_range(File, CharA, CharZ, Style)]),
+		send_tracer(GUI, show_range(File, CharA, CharZ, Style)),
+		(   clause_property(ClauseRef, erased)
+		->  send_tracer(GUI,
+				report(warning,
+				       'Running erased clause; \
+				       source location may be incorrect'))
+		;   true
+		)
+	    ;	prolog_frame_attribute(GUI, Frame, goal, Goal),
+		qualify(Goal, QGoal),
+		(   clause(QGoal, _Body, ClauseRef)
+		->  subgoal_position(GUI, ClauseRef, unify, File, CharA, CharZ),
+		    send_tracer(GUI, show_range(File, CharA, CharZ, Style))
+		;   find_source(QGoal, File, Line),
+		    debug('At ~w:~d~n', [File, Line]),
 		    send_tracer(GUI, show_line(File, Line, Style))
 		)
 	    )
@@ -421,30 +420,46 @@ show_source(Frame, Attributes) :-
 	).
 show_source(_, _).
 
+qualify(Goal, Goal) :-
+	functor(Goal, :, 2), !.
+qualify(Goal, user:Goal).
 
-%%	subgoal_position(+Clause, +PortOrPC, -File, -CharA, -CharZ) is det.
+%%	clause_position(+PC) is semidet.
+%
+%	True if the position can be related to a clause.
+
+clause_position(PC) :- integer(PC), !.
+clause_position(exit).
+clause_position(choice(_)).
+
+%%	subgoal_position(+GUI, +Clause, +PortOrPC, -File, -CharA, -CharZ) is det.
 %
 %	Character  range  CharA..CharZ  in  File   is  the  location  to
 %	highlight for the given clause at the given location.
 
-subgoal_position(ClauseRef, unify, File, CharA, CharZ) :- !,
+subgoal_position(_, ClauseRef, unify, File, CharA, CharZ) :- !,
 	pce_clause_info(ClauseRef, File, TPos, _),
 	head_pos(ClauseRef, TPos, PosTerm),
 	arg(1, PosTerm, CharA),
 	arg(2, PosTerm, CharZ).
-subgoal_position(ClauseRef, choice, File, CharA, CharZ) :- !,
+subgoal_position(GUI, ClauseRef, choice(CHP), File, CharA, CharZ) :- !,
+	(   prolog_choice_attribute(GUI, CHP, type, jump),
+	    prolog_choice_attribute(GUI, CHP, pc, To)
+	->  debug('Term-position: choice-jump to ~w~n', [To]),
+	    subgoal_position(GUI, ClauseRef, To, File, CharA, CharZ)
+	;   pce_clause_info(ClauseRef, File, TPos, _),
+	    arg(2, TPos, CharA),
+	    CharZ is CharA + 1		% i.e. select the dot.
+	).
+subgoal_position(_, ClauseRef, exit, File, CharA, CharZ) :- !,
 	pce_clause_info(ClauseRef, File, TPos, _),
 	arg(2, TPos, CharA),
 	CharZ is CharA + 1.		% i.e. select the dot.
-subgoal_position(ClauseRef, exit, File, CharA, CharZ) :- !,
-	pce_clause_info(ClauseRef, File, TPos, _),
-	arg(2, TPos, CharA),
-	CharZ is CharA + 1.		% i.e. select the dot.
-subgoal_position(ClauseRef, fail, File, CharA, CharZ) :- !,
+subgoal_position(_, ClauseRef, fail, File, CharA, CharZ) :- !,
 	subgoal_position(ClauseRef, exit, File, CharA, CharZ).
-subgoal_position(ClauseRef, exception, File, CharA, CharZ) :- !,
+subgoal_position(_, ClauseRef, exception, File, CharA, CharZ) :- !,
 	subgoal_position(ClauseRef, exit, File, CharA, CharZ).
-subgoal_position(ClauseRef, PC, File, CharA, CharZ) :-
+subgoal_position(_, ClauseRef, PC, File, CharA, CharZ) :-
 	pce_clause_info(ClauseRef, File, TPos, _),
 	(   '$clause_term_position'(ClauseRef, PC, List)
 	->  debug('Term-position: for ref=~w at PC=~w: ~w~n',
@@ -473,12 +488,12 @@ head_pos(_, term_position(_, _, _, _, [HPos,_]), HPos).
 %	warning, ((a,b),c)) --> compiled to (a, (b, c))!!!  We try to correct
 %	that in clause.pl.  This is work in progress.
 
-find_subgoal([], Pos, Pos).
 find_subgoal([A|T], term_position(_, _, _, _, PosL), SPos) :-
-	nth1(A, PosL, Pos),
+	nth1(A, PosL, Pos), !,
 	find_subgoal(T, Pos, SPos).
-find_subgoal([1|T], brace_term_position(_,_,Pos), SPos) :-
+find_subgoal([1|T], brace_term_position(_,_,Pos), SPos) :- !,
 	find_subgoal(T, Pos, SPos).
+find_subgoal(_, Pos, Pos).
 
 
 		 /*******************************
@@ -498,7 +513,8 @@ find_subgoal([1|T], brace_term_position(_,_,Pos), SPos) :-
 %	@tbd	Synchronise with send_pce/1 and in_debug_thread/2.
 
 action(Action) :-
-	thread_self(main), !,
+	pce_thread(Pce),
+	thread_self(Pce), !,
 	get_tracer(action, Action0),
 	debug('Got action ~w~n', [Action0]),
 	action(Action0, Action).
@@ -506,8 +522,11 @@ action(Action) :-
 	send_tracer(prepare_action),
 	repeat,
 	debug(' ---> action: wait~n', []),
-	(   repeat,
-	    catch(thread_get_message('$trace'(Result)), E, wait_error(E))
+	(   thread_self(Me),
+	    thread_debug_queue(Me, Queue),
+	    repeat,
+	    catch(thread_get_message(Queue, '$trace'(Result)),
+		  E, wait_error(E))
 	->  true
 	;   debug('thread_get_message() failed; retrying ...~n'),
 	    fail
@@ -526,7 +545,7 @@ action(Action) :-
 %	thread_get_message/1 can only fail due   to  signals throwing an
 %	exception. For example,  if  the  traced   goal  is  guarded  by
 %	call_with_time_limit/2. Here we  print  the   message  and  keep
-%	waiting. Note that this causes the  system   to  be lost for the
+%	waiting. Note that this causes the  signal   to  be lost for the
 %	application.
 %
 %	@tbd	Allow passing the error to the application
@@ -549,7 +568,8 @@ run_in_debug_thread(Goal, GVars, Caller) :-
 	;   Result = false
 	),
 	debug(' ---> run_in_debug_thread: send ~p~n', [Result]),
-	thread_send_message(Caller, '$trace'(Result)).
+	thread_debug_queue(Caller, Queue),
+	thread_send_message(Queue, '$trace'(Result)).
 
 action(break, Action) :- !,
 	break,
@@ -614,10 +634,17 @@ stack_frames(Depth, F, PC, Frames) :-
 	;   RestFrames = []
 	).
 
-%	choice_frames(+Max, +CHP, +MinLevel-MaxLevel, -Frames)
+%%	choice_frames(+Max, +CHP, +MinLevel-MaxLevel, -Frames) is det.
+%
+%	Frames is a list of frames that hold choice-points.
+%
+%	@param Max is the maximum number of choicepoints returned
+%	@param CHP is the initial choicepoint
+%	@param MinLevel-MaxLevel is the depth-range we consider.
+%	       Currently, MaxLevel is ignored (see in_range/2).
 
 choice_frames(_, none, _, _, []) :- !.
-choice_frames(Max, CHP, Range, Seen, [frame(Frame, choice)|Frames]) :-
+choice_frames(Max, CHP, Range, Seen, [frame(Frame, choice(CH))|Frames]) :-
 	Max > 0,
 	earlier_choice(CHP, CH),
 	visible_choice(CH),
@@ -632,16 +659,17 @@ choice_frames(Max, CHP, Range, Seen, [frame(Frame, choice)|Frames]) :-
 	).
 choice_frames(_, _, _, _, []).
 
-%	earlier_choice(+Here, -Visible)
+%%	earlier_choice(+Here, -Visible) is nondet.
 %
-%	Return earliers choices on backgtracking.
+%	Visible is an older choicepoint  than   Here.  Older choices are
+%	returned on backtracking.
 
 earlier_choice(CHP, CHP).
 earlier_choice(CHP, Next) :-
 	prolog_choice_attribute(CHP, parent, Parent),
 	earlier_choice(Parent, Next).
 
-%	visible_choice(+CHP)
+%%	visible_choice(+CHP) is semidet.
 %
 %	A visible choice is a choice-point that realises a real choice
 %	and is created by a visible frame.
@@ -789,5 +817,4 @@ clustered_binding([B|BR], [B|BT], Value, C) :-
 	clustered_binding(BR, BT, Value, C).
 
 
-:- initialization
-   set_prolog_flag(gui_tracer, true).
+:- create_prolog_flag(gui_tracer, true, []).
