@@ -1,4 +1,4 @@
-/*  $Id: goodies.c,v 1.42 2007/01/18 12:13:22 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -32,7 +32,7 @@
 #include <unistd.h>
 #endif
 
-/* (AA)	isqrt(a).  Returns the square root of a as an integer.  The 
+/* (AA)	isqrt(a).  Returns the square root of a as an integer.  The
 	algorithm only uses bit-shifts (multiplication by a power of 2) and
 	3.5 integer multiplications on average.
  */
@@ -246,7 +246,7 @@ status
 streq_ignore_case(char *s1, char *s2)
 { while( tolower(*s1) == tolower(*s2) && *s2 )
    s1++, s2++;
-  
+
   return *s1 == EOS && *s2 == EOS;
 }
 
@@ -353,7 +353,7 @@ characterName(Any chr)
 	buf[l+1] = EOS;
       }
   }
-  
+
   return WCToName(buf, wcslen(buf));
 }
 
@@ -416,13 +416,13 @@ vwritef(char *fm, va_list args)
 void
 writef(char *fm, ...)
 { va_list args;
-  
+
   va_start(args, fm);
   vwritef(fm, args);
   va_end(args);
 }
 
-  
+
 #define NOT_SET INT_MAX
 #define Put(c) if ( !(*out)(closure, c) ) return FALSE;
 #define PutString(s) do { const char *_s=(s); \
@@ -773,7 +773,7 @@ scanstr(char *str, char *fmt, Any *r)
 	  s++;
 	} else
 	  length = 0;
-	
+
 	switch(*s)			/* conversion char */
     	{ case 'u':
 	    if ( !supress )
@@ -824,7 +824,7 @@ scanstr(char *str, char *fmt, Any *r)
 	    if ( !supress )
 	    { types[argn] = T_CHAR;
 	      ptrs[argn]  = (void *)alloca(sizeof(char));
-	      argn++; 
+	      argn++;
 	    }
 	    s++;
 	    continue;
@@ -1065,7 +1065,7 @@ static struct timeval epoch;
 void
 initMClock()
 { gettimeofday(&epoch, NULL);
-} 
+}
 
 
 unsigned long
@@ -1086,7 +1086,7 @@ static struct _timeb epoch;
 void
 initMClock()
 { _ftime(&epoch);
-} 
+}
 
 
 unsigned long
@@ -1106,18 +1106,36 @@ mclock()
 		*            SLEEP		*
 		********************************/
 
-#if defined(__WATCOMC__)
-#include <dos.h>
+#if !defined(MSLEEP_DONE) && defined(HAVE_NANOSLEEP)
+#define MSLEEP_DONE 1
+#include <errno.h>
 
 void
 msleep(int time)
-{ if ( time > 0 )
-    delay(time);
+{ struct timespec req;
+  int rc;
+
+  if ( time < 0 )
+    return;
+
+  DEBUG(NAME_flash,
+	Cprintf("nanosleep() %d milliseconds ...\n", time));
+
+  req.tv_sec = time/1000;
+  req.tv_nsec = (time%1000)*1000000;
+
+  do
+  { rc = nanosleep(&req, &req);
+  } while ( rc == -1 && errno == EINTR );
+
+  DEBUG(NAME_flash, Cprintf("ok\n"));
 }
 
-#else  /* !defined(__WATCOMC__) */
+#endif
+
+#if !defined(MSLEEP_DONE) && defined(HAVE_SELECT)
+#define MSLEEP_DONE 1
 
-#ifdef HAVE_SELECT
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
@@ -1129,7 +1147,7 @@ msleep(int time)
   timeout.tv_usec = (time%1000)*1000;
 
   DEBUG(NAME_flash,
-	Cprintf("waiting %d milliseconds ...", time));
+	Cprintf("waiting %d milliseconds ...\n", time));
 
 #ifdef SELLIST
   { SELLIST(1,1) readbits = {0, 0};
@@ -1139,13 +1157,15 @@ msleep(int time)
     select(0, &readbits, &writebits, &exceptbits, &timeout);
   }
 #else
-  select(32, NULL, NULL, NULL, &timeout);
+  select(0, NULL, NULL, NULL, &timeout);
 #endif
 
   DEBUG(NAME_flash, Cprintf("ok\n"));
 }
 
-#else /*HAVE_SELECT*/
+#endif
+
+#ifndef MSLEEP_DONE
 
 extern void ws_msleep(int time);
 
@@ -1154,8 +1174,7 @@ msleep(int time)
 { ws_msleep(time);
 }
 
-#endif /*HAVE_SELECT*/
-#endif /*__WATCOMC__*/
+#endif
 
 		 /*******************************
 		 *	 ASSERT() SUPPORT	*
@@ -1201,7 +1220,7 @@ at_pce_exit(atexit_function f, int flags)
       { h->next = NULL;
 	atexit_tail->next = h;
 	atexit_tail = h;
-      }    
+      }
     }
   }
 }
