@@ -39,6 +39,10 @@
 
 install_t install_zlib4pl(void);
 
+static functor_t FUNCTOR_error2;	/* error(Formal, Context) */
+static functor_t FUNCTOR_type_error2;	/* type_error(Term, Expected) */
+static functor_t FUNCTOR_domain_error2;	/* domain_error(Term, Expected) */
+
 static atom_t ATOM_format;		/* format(Format) */
 static atom_t ATOM_level;		/* level(Int) */
 static atom_t ATOM_close_parent;	/* close_parent(Bool) */
@@ -52,6 +56,84 @@ static int debuglevel = 0;
 #define DEBUG(n, g) (void)0
 #endif
 
+		 /*******************************
+		 *	       ERRORS		*
+		 *******************************/
+
+static int
+type_error(term_t actual, const char *expected)
+{ term_t ex;
+
+  if ( (ex = PL_new_term_ref()) &&
+       PL_unify_term(ex,
+		     PL_FUNCTOR, FUNCTOR_error2,
+		       PL_FUNCTOR, FUNCTOR_type_error2,
+		         PL_CHARS, expected,
+		         PL_TERM, actual,
+		       PL_VARIABLE) )
+    return PL_raise_exception(ex);
+
+  return FALSE;
+}
+
+
+static int
+domain_error(term_t actual, const char *domain)
+{ term_t ex;
+
+  if ( (ex = PL_new_term_ref()) &&
+       PL_unify_term(ex,
+		     PL_FUNCTOR, FUNCTOR_error2,
+		       PL_FUNCTOR, FUNCTOR_domain_error2,
+		         PL_CHARS, domain,
+		         PL_TERM, actual,
+		       PL_VARIABLE) )
+  return PL_raise_exception(ex);
+
+  return FALSE;
+}
+
+
+static int
+instantiation_error(void)
+{ term_t ex;
+
+  if ( (ex = PL_new_term_ref()) &&
+       PL_unify_term(ex,
+		     PL_FUNCTOR, FUNCTOR_error2,
+		       PL_CHARS, "instantiation_error",
+		       PL_VARIABLE) )
+    return PL_raise_exception(ex);
+
+  return FALSE;
+}
+
+
+static int
+get_atom_ex(term_t t, atom_t *a)
+{ if ( PL_get_atom(t, a) )
+    return TRUE;
+
+  return type_error(t, "atom");
+}
+
+static int
+get_int_ex(term_t t, int *i)
+{ if ( PL_get_integer(t, i) )
+    return TRUE;
+
+  return type_error(t, "integer");
+}
+
+static int
+get_bool_ex(term_t t, int *i)
+{ if ( PL_get_bool(t, i) )
+    return TRUE;
+
+  return type_error(t, "boolean");
+}
+
+
 		 /*******************************
 		 *	       TYPES		*
 		 *******************************/
@@ -292,23 +374,18 @@ called with data in the buffer. It moves the remaining data to the start
 of the stream buffer and tries to read more data into the stream.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static void
-sync_stream(z_context *ctx)
-{ ctx->stream->bufp   = (char*)ctx->zstate.next_in;
-}
-
 static int
 read_more(z_context *ctx)
 { int c;
 
-  DEBUG(1, Sdprintf("Short input, read_more()\n"));
-
+  ctx->stream->bufp   = (char*)ctx->zstate.next_in;
   ctx->stream->limitp =	ctx->stream->bufp + ctx->zstate.avail_in;
 
   if ( (c=S__fillbuf(ctx->stream)) != EOF )
   { Sungetc(c, ctx->stream);
     ctx->zstate.next_in  = (Bytef*)ctx->stream->bufp;
     ctx->zstate.avail_in = (long)(ctx->stream->limitp - ctx->stream->bufp);
+    ctx->stream->bufp    = ctx->stream->limitp;
 
     return 0;
   }
@@ -330,6 +407,7 @@ zread(void *handle, char *buf, size_t size)
     { ctx->zstate.next_in  = (Bytef*)ctx->stream->bufp;
       ctx->zstate.avail_in = (long)(ctx->stream->limitp - ctx->stream->bufp);
       DEBUG(1, Sdprintf("Set avail_in to %d\n", ctx->zstate.avail_in));
+      ctx->stream->bufp    = ctx->stream->limitp; /* empty buffer */
     }
   }
 
@@ -349,12 +427,7 @@ zread(void *handle, char *buf, size_t size)
       { int rc;
 
 	if ( (rc=read_more(ctx)) < 0 )
-	{ if ( !Sferror(ctx->stream) )
-	  { p = NULL;
-	    break;
-	  }
 	  return -1;
-	}
       }
     }
 
@@ -365,13 +438,9 @@ zread(void *handle, char *buf, size_t size)
       DEBUG(1, Sdprintf("Skipped gzip header (%d bytes)\n", m));
       ctx->zstate.next_in = p;
       ctx->zstate.avail_in -= m;
-      sync_stream(ctx);
 
 					/* init without header */
-      rc = inflateInit2(&ctx->zstate, -MAX_WBITS);
-      sync_stream(ctx);
-
-      switch( rc )
+      switch(inflateInit2(&ctx->zstate, -MAX_WBITS))
       { case Z_OK:
 	  ctx->initialized = TRUE;
 	  ctx->crc = crc32(0L, Z_NULL, 0);
@@ -386,10 +455,7 @@ zread(void *handle, char *buf, size_t size)
 	  return -1;
       }
     } else
-    { rc = inflateInit(&ctx->zstate);
-      sync_stream(ctx);
-
-      switch( rc )
+    { switch(inflateInit(&ctx->zstate))
       { case Z_OK:
 	  ctx->format = F_DEFLATE;
 	  ctx->initialized = TRUE;
@@ -410,25 +476,27 @@ zread(void *handle, char *buf, size_t size)
     while( (rc=gz_skip_footer(ctx)) == -2 )
     { int rc2;
 
-      sync_stream(ctx);
       if ( (rc2=read_more(ctx)) < 0 )
 	return -1;
     }
 
-    sync_stream(ctx);
-
     if ( rc == 0 )
-    { return 0;			/* EOF */
+    { int avail = ctx->zstate.avail_in;
+
+					/* copy back unprocessed bytes */
+      DEBUG(1, Sdprintf("GZIP footer ok; copying %d bytes back\n", avail));
+      memmove(ctx->stream->buffer, ctx->zstate.next_in, avail);
+      ctx->stream->bufp   = ctx->stream->buffer;
+      ctx->stream->limitp = ctx->stream->bufp + avail;
+
+      return 0;			/* EOF */
     } else
     { DEBUG(1, Sdprintf("GZIP CRC/length error\n"));
       return -1;
     }
   }
 
-  rc = inflate(&ctx->zstate, Z_NO_FLUSH);
-  sync_stream(ctx);
-
-  switch( rc )
+  switch((rc=inflate(&ctx->zstate, Z_NO_FLUSH)))
   { case Z_OK:
     case Z_STREAM_END:
     { long n = (long)(size - ctx->zstate.avail_out);
@@ -636,32 +704,32 @@ pl_zopen(term_t org, term_t new, term_t options)
     term_t arg = PL_new_term_ref();
 
     if ( !PL_get_name_arity(head, &name, &arity) || arity != 1 )
-      return PL_type_error("option", head);
+      return type_error(head, "option");
     _PL_get_arg(1, head, arg);
 
     if ( name == ATOM_format )
     { atom_t a;
 
-      if ( !PL_get_atom_ex(arg, &a) )
+      if ( !get_atom_ex(arg, &a) )
 	return FALSE;
       if ( a == ATOM_gzip )
 	fmt = F_GZIP;
       else if ( a == ATOM_deflate )
 	fmt = F_DEFLATE;
       else
-	return PL_domain_error("compression_format", arg);
+	return domain_error(arg, "compression_format");
     } else if ( name == ATOM_level )
-    { if ( !PL_get_integer_ex(arg, &level) )
+    { if ( !get_int_ex(arg, &level) )
 	return FALSE;
       if ( level < 0 || level > 9 )
-	return PL_domain_error("compression_level", arg);
+	return domain_error(arg, "compression_level");
     } else if ( name == ATOM_close_parent )
-    { if ( !PL_get_bool_ex(arg, &close_parent) )
+    { if ( !get_bool_ex(arg, &close_parent) )
 	return FALSE;
     }
   }
-  if ( !PL_get_nil_ex(tail) )
-    return FALSE;
+  if ( !PL_get_nil(tail) )
+    return type_error(tail, "list");
 
   if ( !PL_get_stream_handle(org, &s) )
     return FALSE;			/* Error */
@@ -704,7 +772,7 @@ pl_zopen(term_t org, term_t new, term_t options)
   } else
   { ctx->close_parent = FALSE;
     Sclose(s2);
-    return PL_instantiation_error(new);	/* actually, over instantiation */
+    return instantiation_error();
   }
 }
 
@@ -724,7 +792,11 @@ zdebug(term_t level)
 
 install_t
 install_zlib4pl(void)
-{ ATOM_format       = PL_new_atom("format");
+{ FUNCTOR_error2        = MKFUNCTOR("error", 2);
+  FUNCTOR_type_error2   = MKFUNCTOR("type_error", 2);
+  FUNCTOR_domain_error2 = MKFUNCTOR("domain_error", 2);
+
+  ATOM_format       = PL_new_atom("format");
   ATOM_level        = PL_new_atom("level");
   ATOM_close_parent = PL_new_atom("close_parent");
   ATOM_gzip	    = PL_new_atom("gzip");
