@@ -29,15 +29,21 @@ This module is a test-frame for testing built-in predicates.
 
 :- module(test_bips, [test_bips/0]).
 :- use_module(library(plunit)).
+:- use_module(library(when)).
 
 test_bips :-
-	run_tests([bips,bips_occurs_check_error,coroutining]).
+	run_tests([ bips,
+		    bips_occurs_check_error,
+		    coroutining,
+		    eq
+		  ]).
 
 has_occurs_check_flag :-
 	catch(current_prolog_flag(occurs_check, _), _, fail).
 
 
 :- begin_tests(bips).
+
 test(iso_8_3_10_4,[]) :-
        ground(3).
 test(iso_8_3_10_4,[fail]) :-
@@ -152,9 +158,9 @@ test(when2,[error(instantiation_error)]) :-
 	when((_,_),1=2).
 test(when3,[error(instantiation_error)]) :-
 	when((nonvar(_),_),1=2).
-%test(when4_inf,[sto(infinite_trees), blocked(trop)]) :-
-%	C=(C,C),
-%	when(C,1=2).
+test(when4_inf,[sto(rational_trees), error(type_error(_,_))]) :-
+	C=(C,C),
+	when(C,1=2).
 test(when5_r,[true(X==2)]) :-
 	when(ground(g),X=2).
 test(when6,[error(domain_error(_,_))]) :-
@@ -167,3 +173,14 @@ test(when8,[fail]) :-
 	when(ground(g),fail).
 
 :- end_tests(coroutining).
+
+
+:- begin_tests(eq).
+
+a :-
+	A == A.
+
+test(eq_ff, true) :-
+	a.
+
+:- end_tests(eq).
