@@ -148,8 +148,7 @@ PL_get_text__LD(term_t l, PL_chars_t *text, int flags ARG_LD)
       { size_t sz = mpz_sizeinbase(n.value.mpz, 10) + 2;
 	Buffer b  = findBuffer(BUF_RING);
 
-	if ( !growBuffer(b, sz) )
-	  outOfCore();
+	growBuffer(b, sz);
 	mpz_get_str(b->base, 10, n.value.mpz);
 	b->top = b->base + strlen(b->base);
 	text->text.t  = baseBuffer(b, char);
@@ -500,7 +499,7 @@ PL_unify_text_range(term_t term, PL_chars_t *text,
       int c;
 
       if ( text->encoding == ENC_ISO_LATIN_1 )
-	c = text->text.t[offset]&0xff;
+	c = text->text.t[offset];
       else
 	c = text->text.w[offset];
 
