@@ -24,6 +24,7 @@
 
 /*#define O_DEBUG 1*/
 #include "pl-incl.h"
+#include "pl-inline.h"
 #undef LD
 #define LD LOCAL_LD
 
@@ -76,7 +77,7 @@ mp_alloc(size_t bytes)
   if ( TOO_BIG_GMP(bytes) ||
        !(mem = malloc(sizeof(mp_mem_header)+bytes)) )
   { gmp_too_big();
-    pl_abort(ABORT_FATAL);
+    abortProlog(ABORT_THROW);
     return NULL;			/* make compiler happy */
   }
 
@@ -110,7 +111,7 @@ mp_realloc(void *ptr, size_t oldsize, size_t newsize)
   if ( TOO_BIG_GMP(newsize) ||
        !(newmem = realloc(oldmem, sizeof(mp_mem_header)+newsize)) )
   { gmp_too_big();
-    pl_abort(ABORT_FATAL);
+    abortProlog(ABORT_THROW);
     return NULL;			/* make compiler happy */
   }
 
@@ -203,24 +204,40 @@ globalMPZ() pushes an mpz type GMP  integer   onto  the local stack. The
 saved version is the _mp_size field, followed by the limps.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static word
-globalMPZ(mpz_t mpz)
-{ GET_LD
-  Word p;
-  word r;
-
-  size_t size = sizeof(mp_limb_t)*abs(mpz->_mp_size);
+static size_t
+mpz_wsize(mpz_t mpz, size_t *s)
+{ size_t size = sizeof(mp_limb_t)*abs(mpz->_mp_size);
   size_t wsz  = (size+sizeof(word)-1)/sizeof(word);
-  word m      = mkIndHdr(wsz+1, TAG_INTEGER);
+
+  if ( s )
+    *s = size;
+
+  return wsz;
+}
+
+
+static int
+globalMPZ(Word at, mpz_t mpz, int flags ARG_LD)
+{ Word p;
+  size_t size;
+  size_t wsz = mpz_wsize(mpz, &size);
+  word m     = mkIndHdr(wsz+1, TAG_INTEGER);
 
   if ( wsizeofInd(m) != wsz+1 )
   { PL_error(NULL, 0, NULL, ERR_REPRESENTATION, ATOM_integer);
     return 0;
   }
 
-  p = allocGlobal(wsz+3);
-  r = consPtr(p, TAG_INTEGER|STG_GLOBAL);
+  if ( !hasGlobalSpace(wsz+3) )
+  { int rc = ensureGlobalSpace(wsz+3, flags);
 
+    if ( rc != TRUE )
+      return rc;
+  }
+  p = gTop;
+  gTop += wsz+3;
+
+  *at = consPtr(p, TAG_INTEGER|STG_GLOBAL);
 
   *p++     = m;
   p[wsz]   = 0L;			/* pad out */
@@ -228,7 +245,7 @@ globalMPZ(mpz_t mpz)
   *p++     = (word)mpz->_mp_size;
   memcpy(p, mpz->_mp_d, size);
 
-  return r;
+  return TRUE;
 }
 
 
@@ -558,6 +575,28 @@ initGMP()
   }
 }
 
+
+void
+cleanupGMP()
+{ if ( GD->gmp.initialised )
+  { GD->gmp.initialised = FALSE;
+
+#ifdef O_MY_GMP_ALLOC
+    if ( !GD->gmp.keep_alloc_functions )
+      mp_set_memory_functions(NULL, NULL, NULL);
+#endif
+    mpz_clear(MPZ_MIN_TAGGED);
+    mpz_clear(MPZ_MAX_TAGGED);
+    mpz_clear(MPZ_MIN_PLINT);
+    mpz_clear(MPZ_MAX_PLINT);
+#if SIZEOF_LONG < SIZEOF_VOIDP
+    mpz_clear(MPZ_MIN_LONG);
+    mpz_clear(MPZ_MAX_LONG);
+#endif
+  }
+}
+
+
 		 /*******************************
 		 *	   NUMBER HANDLING      *
 		 *******************************/
@@ -593,15 +632,15 @@ most compact representation, which is  essential   to  make unify() work
 without any knowledge of the represented data.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static word
-put_mpz(mpz_t mpz)
+static int
+put_mpz(Word at, mpz_t mpz, int flags ARG_LD)
 { int64_t v;
 
-    DEBUG(2,
-	  { char buf[256];
-	    Sdprintf("put_mpz(%s)\n",
-		     mpz_get_str(buf, 10, mpz));
-	  });
+  DEBUG(2,
+	{ char buf[256];
+	  Sdprintf("put_mpz(%s)\n",
+		   mpz_get_str(buf, 10, mpz));
+	});
 
 #if SIZEOF_LONG < SIZEOF_VOIDP
   if ( mpz_cmp(mpz, MPZ_MIN_LONG) >= 0 &&
@@ -612,63 +651,101 @@ put_mpz(mpz_t mpz)
 #endif
   { long v = mpz_get_si(mpz);
 
-    return consInt(v);
-  } else if ( mpz_to_int64(mpz, &v) )
-  { GET_LD
+    if ( !hasGlobalSpace(0) )		/* ensure we have room for bindConst */
+    { int rc = ensureGlobalSpace(0, flags);
 
-#if SIZEOF_LONG < SIZEOF_VOIDP
-    return makeNum(v);
-#else
-    return globalLong(v PASS_LD);
-#endif
+      if ( rc != TRUE )
+	return rc;
+    }
+
+    *at = consInt(v);
+    return TRUE;
+  } else if ( mpz_to_int64(mpz, &v) )
+  { return put_int64(at, v, flags PASS_LD);
   } else
-  { return globalMPZ(mpz);
+  { return globalMPZ(at, mpz, flags PASS_LD);
   }
 }
 
 #endif /*O_GMP*/
 
-word
-put_number__LD(Number n ARG_LD)
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+put_number()   translates   a   number   structure   into   its   Prolog
+representation and ensures there  is  enough   space  for  a  subsequent
+bindConst() call. Note that `at' must point   to  an address that is not
+affected by GC/shift.  The intented scenario is:
+
+  { word c;
+
+    if ( (rc=put_number(&c, n, ALLOW_GC PASS_LD)) == TRUE )
+      bindConst(<somewhere>, c);
+    ...
+  }
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+int
+put_number(Word at, Number n, int flags ARG_LD)
 { switch(n->type)
   { case V_INTEGER:
     { word w = consInt(n->value.i);
 
       if ( valInt(w) == n->value.i )
-	return w;
+      { if ( !hasGlobalSpace(0) )
+	{ int rc = ensureGlobalSpace(0, flags);
 
-      return globalLong(n->value.i PASS_LD);
+	  if ( rc != TRUE )
+	    return rc;
+	}
+
+	*at = w;
+        return TRUE;
+      }
+
+      return put_int64(at, n->value.i, flags PASS_LD);
     }
 #ifdef O_GMP
     case V_MPZ:
-      return put_mpz(n->value.mpz);
+      return put_mpz(at, n->value.mpz, flags PASS_LD);
     case V_MPQ:
     { if ( mpz_cmp_ui(mpq_denref(n->value.mpq), 1L) == 0 )
-      { return put_mpz(mpq_numref(n->value.mpq));
+      { return put_mpz(at, mpq_numref(n->value.mpq), flags PASS_LD);
       } else
       { word num, den;
 	Word p;
+	size_t req = ( mpz_wsize(mpq_numref(n->value.mpq), NULL)+3 +
+		       mpz_wsize(mpq_denref(n->value.mpq), NULL)+3 + 3 );
 
-	if ( !(num=put_mpz(mpq_numref(n->value.mpq))) ||
-	     !(den=put_mpz(mpq_denref(n->value.mpq))) )
+	if ( !hasGlobalSpace(req) )
+	{ int rc = ensureGlobalSpace(req, flags);
+
+	  if ( rc != TRUE )
+	    return rc;
+	}
+
+	if ( !(put_mpz(&num, mpq_numref(n->value.mpq), 0 PASS_LD)) ||
+	     !(put_mpz(&den, mpq_denref(n->value.mpq), 0 PASS_LD)) )
 	  fail;
 
-	p = allocGlobal(3);
+	p = gTop;
+	gTop += 3;
+	assert(gTop <= gMax);
+
 
 	p[0] = FUNCTOR_rdiv2;
 	p[1] = num;
 	p[2] = den;
 
-	return consPtr(p, TAG_COMPOUND|STG_GLOBAL);
+	*at = consPtr(p, TAG_COMPOUND|STG_GLOBAL);
+	return TRUE;
       }
     }
 #endif
     case V_FLOAT:
-      return globalFloat(n->value.f);
+      return put_double(at, n->value.f, flags PASS_LD);
   }
 
   assert(0);
-  return 0L;
+  return FALSE;
 }
 
 
@@ -681,9 +758,10 @@ PL_unify_number(term_t t, Number n)
 
   if ( canBind(*p) )
   { word w;
+    int rc;
 
-    if ( !(w = put_number(n)) )
-      fail;
+    if ( (rc=put_number(&w, n, ALLOW_GC PASS_LD)) != TRUE )
+      return raiseStackOverflow(rc);
 
     p = valTermRef(t);			/* put_number can shift the stacks */
     deRef(p);
@@ -712,9 +790,10 @@ PL_unify_number(term_t t, Number n)
 #ifdef O_GMP
     case V_MPQ:
     { word w;
+      int rc;
 
-      if ( !(w=put_number(n)) )
-	fail;
+      if ( (rc=put_number(&w, n, ALLOW_GC PASS_LD)) != TRUE )
+	return raiseStackOverflow(rc);
 
       return _PL_unify_atomic(t, w);
     }
@@ -764,7 +843,7 @@ PL_get_number(term_t t, Number n)
 		 *	     PROMOTION		*
 		 *******************************/
 
-void
+int
 promoteToFloatNumber(Number n)
 { switch(n->type)
   { case V_INTEGER:
@@ -775,6 +854,9 @@ promoteToFloatNumber(Number n)
     case V_MPZ:
     { double val = mpz_get_d(n->value.mpz);
 
+      if ( !check_float(val) )
+	return FALSE;
+
       clearNumber(n);
       n->value.f = val;
       n->type = V_FLOAT;
@@ -783,6 +865,9 @@ promoteToFloatNumber(Number n)
     case V_MPQ:
     { double val = mpq_get_d(n->value.mpq);
 
+      if ( !check_float(val) )
+	return FALSE;
+
       clearNumber(n);
       n->value.f = val;
       n->type = V_FLOAT;
@@ -792,6 +877,8 @@ promoteToFloatNumber(Number n)
     case V_FLOAT:
       break;
   }
+
+  return TRUE;
 }
 
 
@@ -999,7 +1086,15 @@ PL_unify_mpq(term_t t, mpq_t mpq)
 		 *               WIN64		*
 		 *******************************/
 
-#ifdef WIN64
+
+#if defined(WIN64) && _MSC_VER <= 1400 && !defined(_CRT_ASSEMBLY_VERSION)
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+"#if (_MSC_VER <= 1400)" should suffice,   but  although both the VS2005
+(VC8) and the Microsoft Server 2003   R2 (VC8 SDK) define _MSC_VER=1400,
+VC8 SDK does not define the below functions, while VC8 does... The macro
+below distinguishes the two.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 size_t
 strnlen(const char *s, size_t maxlen)
 { size_t len = 0;
@@ -1010,12 +1105,10 @@ strnlen(const char *s, size_t maxlen)
   return len;
 }
 
-
 void
 __GSHandlerCheck()
 {
 }
-
 #endif
 
 #endif /*O_GMP*/
