@@ -80,7 +80,8 @@ typedef struct _opdef			/* predefined and enumerated */
 
 static void
 copyOperatorSymbol(Symbol s)
-{ operator *op = s->value;
+{ GET_LD
+  operator *op = s->value;
   operator *o2 = allocHeap(sizeof(*o2));
 
   *o2 = *op;
@@ -89,7 +90,8 @@ copyOperatorSymbol(Symbol s)
 
 static void
 freeOperatorSymbol(Symbol s)
-{ operator *op = s->value;
+{ GET_LD
+  operator *op = s->value;
 
   PL_unregister_atom((atom_t) s->name);
   freeHeap(op, sizeof(*op));
@@ -116,9 +118,10 @@ Define operator in some table.  See   discussion  above for the relevant
 tables.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static void
-defOperator(Module m, atom_t name, int type, int priority)
-{ Symbol s;
+static int
+defOperator(Module m, atom_t name, int type, int priority, int force)
+{ GET_LD
+  Symbol s;
   operator *op;
   int t = (type & OP_MASK);		/* OP_PREFIX, ... */
 
@@ -130,6 +133,20 @@ defOperator(Module m, atom_t name, int type, int priority)
 
   assert(t>=OP_PREFIX && t<=OP_POSTFIX);
 
+  if ( !force && !SYSTEM_MODE )
+  { if ( name == ATOM_comma || name == ATOM_nil || name == ATOM_curl ||
+	 (name == ATOM_bar && ((t&OP_MASK) != OP_INFIX ||
+			       (priority < 1001 && priority != 0))) )
+    { GET_LD
+      atom_t action = (name == ATOM_comma ? ATOM_modify : ATOM_create);
+      term_t t = PL_new_term_ref();
+
+      PL_put_atom(t, name);
+      return PL_error(NULL, 0, NULL, ERR_PERMISSION,
+		      action, ATOM_operator, t);
+    }
+  }
+
   LOCK();
   if ( !m->operators )
     m->operators = newOperatorTable();
@@ -138,7 +155,7 @@ defOperator(Module m, atom_t name, int type, int priority)
   { op = s->value;
   } else if ( priority < 0 )
   { UNLOCK();				/* already inherited: do not change */
-    return;
+    return TRUE;
   } else
   { op = allocHeap(sizeof(*op));
 
@@ -157,6 +174,8 @@ defOperator(Module m, atom_t name, int type, int priority)
     addHTable(m->operators, (void *)name, op);
   }
   UNLOCK();
+
+  return TRUE;
 }
 
 
@@ -284,7 +303,6 @@ atomToOperatorType(atom_t atom)
   else if (atom == ATOM_xfx)		return OP_XFX;
   else if (atom == ATOM_xfy)		return OP_XFY;
   else if (atom == ATOM_yfx)		return OP_YFX;
-  else if (atom == ATOM_yfy)		return OP_YFY;
   else if (atom == ATOM_yf)		return OP_YF;
   else if (atom == ATOM_xf)		return OP_XF;
 
@@ -297,7 +315,7 @@ operatorTypeToAtom(int type)
   { NULL_ATOM,
     ATOM_fx, ATOM_fy,
     ATOM_xf, ATOM_yf,
-    ATOM_xfx, ATOM_xfy, ATOM_yfx, ATOM_yfy
+    ATOM_xfx, ATOM_xfy, ATOM_yfx
   };
 
   return opnames[(type>>4)];
@@ -328,11 +346,10 @@ PRED_IMPL("op", 3, op, PL_FA_TRANSPARENT|PL_FA_ISO)
     term_t a = PL_new_term_ref();
 
     PL_put_atom(a, m->name);
-    PL_cons_functor(t, FUNCTOR_colon2, a, name);
-
-    return PL_error(NULL, 0, "system operators are protected",
-		    ERR_PERMISSION, ATOM_redefine, ATOM_operator,
-		    t);
+    return (PL_cons_functor(t, FUNCTOR_colon2, a, name) &&
+	    PL_error(NULL, 0, "system operators are protected",
+		     ERR_PERMISSION, ATOM_redefine, ATOM_operator,
+		     t));
   }
 
   if ( !PL_get_atom_ex(type, &tp) )
@@ -345,12 +362,7 @@ PRED_IMPL("op", 3, op, PL_FA_TRANSPARENT|PL_FA_ISO)
     return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_operator_specifier, type);
 
   if ( PL_get_atom(name, &nm) )
-  { if ( nm == ATOM_comma )
-    { protected:
-      return PL_error(NULL, 0, NULL, ERR_PERMISSION,
-		      ATOM_modify, ATOM_operator, name);
-    }
-    defOperator(m, nm, t, p);
+  { return defOperator(m, nm, t, p, FALSE);
   } else
   { term_t l = PL_copy_term_ref(name);
     term_t e = PL_new_term_ref();
@@ -358,9 +370,8 @@ PRED_IMPL("op", 3, op, PL_FA_TRANSPARENT|PL_FA_ISO)
     while( PL_get_list(l, e, l) )
     { if ( !PL_get_atom(e, &nm) )
 	return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_atom, e);
-      if ( nm == ATOM_comma )
-	goto protected;
-      defOperator(m, nm, t, p);
+      if ( !defOperator(m, nm, t, p, FALSE) )
+	return FALSE;
     }
     if ( !PL_get_nil(l) )
       return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_list, l);
@@ -590,7 +601,8 @@ static const opdef operators[] = {
   OP(ATOM_ifthen,		 OP_XFY, 1050),	/* -> */
   OP(ATOM_softcut,		 OP_XFY, 1050),	/* *-> */
   OP(ATOM_divide,		 OP_YFX, 400),	/* / */
-  OP(ATOM_div,			 OP_YFX, 400),	/* // */
+  OP(ATOM_gdiv,			 OP_YFX, 400),	/* // */
+  OP(ATOM_div,			 OP_YFX, 400),	/* div */
   OP(ATOM_rdiv,			 OP_YFX, 400),	/* rdiv */
   OP(ATOM_and,			 OP_YFX, 500),	/* /\ */
   OP(ATOM_colon,		 OP_XFY, 600),	/* : */
@@ -616,10 +628,9 @@ static const opdef operators[] = {
   OP(ATOM_at_larger_eq,		 OP_XFX, 700),	/* @>= */
   OP(ATOM_backslash,		 OP_FY,	 200),	/* \ */
   OP(ATOM_not_provable,		 OP_FY,	 900),	/* \+ */
-  OP(ATOM_or,			 OP_YFX, 500),	/* \/ */
-  OP(ATOM_bw_xor,		 OP_YFX, 500),	/* >< */
+  OP(ATOM_bitor,		 OP_YFX, 500),	/* \/ */
   OP(ATOM_not_equals,		 OP_XFX, 700),	/* \= */
-  OP(ATOM_not_strickt_equals,	 OP_XFX, 700),	/* \== */
+  OP(ATOM_not_strict_equal,	 OP_XFX, 700),	/* \== */
   OP(ATOM_at_equals,		 OP_XFX, 700),	/* =@= */
   OP(ATOM_at_not_equals,	 OP_XFX, 700),	/* \=@= */
   OP(ATOM_hat,			 OP_XFY, 200),	/* ^ */
@@ -637,6 +648,7 @@ static const opdef operators[] = {
   OP(ATOM_module_transparent,	 OP_FX,	 1150),	/* module_transparent */
   OP(ATOM_multifile,		 OP_FX,	 1150),	/* multifile */
   OP(ATOM_meta_predicate,	 OP_FX,	 1150),	/* meta_predicate */
+  OP(ATOM_public,		 OP_FX,	 1150),	/* public */
   OP(ATOM_xor,			 OP_YFX, 400),	/* xor */
 
   OP(NULL_ATOM,			 0,	 0)
@@ -648,7 +660,7 @@ initOperators(void)
 { const opdef *op;
 
   for( op = operators; op->name; op++ )
-    defOperator(MODULE_system, op->name, op->type, op->priority);
+    defOperator(MODULE_system, op->name, op->type, op->priority, TRUE);
 }
 
 
